package cd.casic.plugin.extension;

import cd.casic.plugin.annotation.BasePluginInformation;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.v3.oas.annotations.media.Schema;
import kotlin.Metadata;
import org.springframework.util.StringUtils;

import java.util.function.Predicate;

import static io.swagger.v3.oas.annotations.media.Schema.RequiredMode.REQUIRED;

/**
 * @Classname ExtensionOperator
 * @Description 扩展类的操作
 * @Date 2025/5/8 15:12
 * @Created by mianbin
 */
public interface ExtensionOperator {

    @Schema(requiredMode = REQUIRED)
    @JsonProperty("apiVersion")
    default String getApiVersion() {
        final var basePluginInformation = getClass().getAnnotation(BasePluginInformation.class);
        if (basePluginInformation == null) {
            return null;
        }
        if (StringUtils.hasText(basePluginInformation.group())) {
            return basePluginInformation.group() + "/" + basePluginInformation.version();
        }
        return basePluginInformation.version();
    }

    @Schema(requiredMode = REQUIRED)
    @JsonProperty("kind")
    default String getKind() {
        final var basePluginInformation = getClass().getAnnotation(BasePluginInformation.class);
        if (basePluginInformation == null) {
            // return null if having no GVK annotation
            return null;
        }
        return basePluginInformation.kind();
    }

    @Schema(requiredMode = REQUIRED, implementation = Metadata.class)
    @JsonProperty("metadata")
    MetadataOperator getMetadata();

    void setApiVersion(String apiVersion);

    void setKind(String kind);

    void setMetadata(MetadataOperator metadata);

    default void groupVersionKind(GroupVersionKind gvk) {
        setApiVersion(gvk.groupVersion().toString());
        setKind(gvk.kind());
    }

    @JsonIgnore
    default GroupVersionKind groupVersionKind() {
        return GroupVersionKind.fromAPIVersionAndKind(getApiVersion(), getKind());
    }

    static <T extends ExtensionOperator> Predicate<T> isNotDeleted() {
        return ext -> ext.getMetadata().getDeletionTimestamp() == null;
    }

    static boolean isDeleted(ExtensionOperator extension) {
        return ExtensionUtil.isDeleted(extension);
    }

}
