//package com.casic.machine.service.impl;
//
//import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
//import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
//import com.casic.commons.exception.ServiceException;
//import com.casic.commons.utils.AliOssUtil;
//import com.casic.machine.entity.MachineInfo;
//import com.casic.machine.entity.SecretKey;
//import com.casic.machine.dto.SecretKeyDto;
//import com.casic.machine.mapper.SecretServiceMapper;
//import com.casic.machine.service.MachineInfoService;
//import com.jayway.jsonpath.internal.Utils;
//import org.junit.jupiter.api.BeforeEach;
//import org.junit.jupiter.api.Test;
//import org.mockito.InjectMocks;
//import org.mockito.Mock;
//import org.mockito.MockitoAnnotations;
//import org.springframework.beans.BeanUtils;
//import org.springframework.core.io.InputStreamResource;
//import org.springframework.http.ResponseEntity;
//import org.springframework.mock.web.MockMultipartFile;
//import org.springframework.test.util.ReflectionTestUtils;
//
//import java.io.ByteArrayInputStream;
//import java.io.IOException;
//import java.util.Collections;
//import java.util.List;
//
//import static org.junit.jupiter.api.Assertions.*;
//import static org.mockito.ArgumentMatchers.any;
//import static org.mockito.Mockito.*;
//
//@SuppressWarnings("unchecked")
//public class SecretKeyServiceImplTest {
//
//    @InjectMocks
//    private SecretKeyServiceImpl secretKeyService;
//
//    @Mock
//    private AliOssUtil aliOssUtil;
//
//    @Mock
//    private SecretServiceMapper secretServiceMapper;
//
//    @Mock
//    private MachineInfoService machineInfoService;
//
//    private SecretKeyDto validSecretKeyDto;
//    private MockMultipartFile mockFile;
//    private final Long TEST_SECRET_KEY_ID = 1L;
//    private final String TEST_FILE_NAME = "test_key.pem";
//    private final String TEST_PATH = "https://bucket.endpoint/test_key.pem";
//
//    @BeforeEach
//    public void setUp() throws IOException {
//        MockitoAnnotations.openMocks(this);
//        validSecretKeyDto = new SecretKeyDto();
//        validSecretKeyDto.setName("Test Key");
//        validSecretKeyDto.setDescription("Test secret key");
//
//        // 创建模拟文件
//        byte[] fileContent = "test content".getBytes();
//        mockFile = new MockMultipartFile(
//                "file",
//                TEST_FILE_NAME,
//                "application/octet-stream",
//                new ByteArrayInputStream(fileContent)
//        );
//
//        // 模拟 OSS 工具返回值
//        when(aliOssUtil.save(any(MockMultipartFile.class))).thenReturn(TEST_FILE_NAME);
//
//    }
//
//    // ======================= 新增密钥测试 =======================
//    @Test
//    void testAddSecretKey_ValidData_ShouldSucceed() throws IOException {
//        // 执行新增
//        boolean result = secretKeyService.addSecretKey(validSecretKeyDto, mockFile);
//
//        // 验证 OSS 保存调用
//        verify(aliOssUtil, times(1)).save(mockFile);
//
//        // 验证实体属性
//        SecretKey savedKey = new SecretKey();
//        BeanUtils.copyProperties(validSecretKeyDto, savedKey);
//        savedKey.setFileName(TEST_FILE_NAME);
//        savedKey.setPath(TEST_PATH);
//
//        // 验证 Mapper 调用
//        verify(secretServiceMapper, times(1)).insert(savedKey);
//        assertTrue(result);
//    }
//
//    @Test
//    void testAddSecretKey_NullFile_ShouldThrowException() {
//        assertThrows(ServiceException.class, () -> {
//            secretKeyService.addSecretKey(validSecretKeyDto, null);
//        }, "文件为空");
//    }
//
//    // ======================= 绑定机器测试 =======================
//    @Test
//    void testBindingMachine_ValidIds_ShouldUpdateMachine() {
//        // 模拟机器列表
//        List<Long> machineIds = Collections.singletonList(1L);
//        when(machineInfoService.listByIds(machineIds)).thenReturn(Collections.singletonList(new MachineInfo()));
//
//        secretKeyService.bindingMachine(TEST_SECRET_KEY_ID, machineIds);
//
//        // 验证机器信息更新
//        verify(machineInfoService, times(1)).listByIds(machineIds);
//        machineIds.forEach(id -> {
//            verify(machineInfoService, times(1)).update(any());
//        });
//    }
//
//    // ======================= 更新密钥测试 =======================
//    @Test
//    void testUpdateSecretKey_WithNewFile_ShouldUpdatePath() throws IOException {
//        MockMultipartFile newFile = new MockMultipartFile(
//                "file",
//                "new_key.pem",
//                "application/octet-stream",
//                new ByteArrayInputStream("new content".getBytes())
//        );
//        when(aliOssUtil.save(newFile)).thenReturn("new_key.pem");
//
//        validSecretKeyDto.setId(TEST_SECRET_KEY_ID);
//        boolean result = secretKeyService.updateSecretKey(validSecretKeyDto, newFile);
//
//        // 验证 OSS 调用和路径更新
//        verify(aliOssUtil, times(1)).save(newFile);
//        SecretKey updatedKey = new SecretKey();
//        BeanUtils.copyProperties(validSecretKeyDto, updatedKey);
//        updatedKey.setFileName("new_key.pem");
//        updatedKey.setPath("https://bucket.endpoint/new_key.pem");
//        verify(secretServiceMapper, times(1)).updateById(updatedKey);
//        assertTrue(result);
//    }
//
//    // ======================= 删除密钥测试 =======================
//    @Test
//    void testDeleteSecretKey_ValidId_ShouldDeleteFileAndRecord() {
//        SecretKey secretKey = new SecretKey();
//        secretKey.setId(TEST_SECRET_KEY_ID);
//        secretKey.setFileName(TEST_FILE_NAME);
//        when(secretServiceMapper.selectById(TEST_SECRET_KEY_ID)).thenReturn(secretKey);
//
//        boolean result = secretKeyService.deleteSecretKey(TEST_SECRET_KEY_ID);
//
//        // 验证 OSS 删除和 Mapper 调用
//        verify(aliOssUtil, times(1)).deleteFile(TEST_FILE_NAME);
//        verify(secretServiceMapper, times(1)).deleteById(TEST_SECRET_KEY_ID);
//        assertTrue(result);
//    }
//
//    // ======================= 列表查询测试 =======================
//    @Test
//    void testListSecretKey_WithNameFilter_ShouldReturnMatchedRecords() {
//        SecretKeyDto queryDto = new SecretKeyDto();
//        queryDto.setName("Test");
//        QueryWrapper<SecretKey> wrapper = new QueryWrapper<>();
//        wrapper.like("name", "Test");
//
//        when(secretServiceMapper.selectPage(any(), any())).thenReturn(new Page<>());
//
//        secretKeyService.listSecretKey(queryDto);
//        verify(secretServiceMapper, times(1)).selectPage(any(), wrapper);
//    }
//
//    // ======================= 文件下载测试 =======================
//    @Test
//    void testDownloadSecretKeyFile_ValidId_ShouldReturnResponseEntity() throws IOException {
//        SecretKey secretKey = new SecretKey();
//        secretKey.setFileName(TEST_FILE_NAME);
//        when(secretServiceMapper.selectById(TEST_SECRET_KEY_ID)).thenReturn(secretKey);
//        InputStreamResource inputStreamResource = new InputStreamResource(
//                new ByteArrayInputStream("test content".getBytes())
//        );
//        when(aliOssUtil.downloadFile(TEST_FILE_NAME)).thenReturn(
//                ResponseEntity.ok(inputStreamResource)
//        );
//
//        ResponseEntity<InputStreamResource> response = secretKeyService.downloadSecretKeyFile(TEST_SECRET_KEY_ID);
//        assertNotNull(response);
//        assertEquals("test content", Utils.toString(response.getBody().getInputStream()));
//    }
//
//    // ======================= 批量删除测试 =======================
//    @Test
//    void testDeleteList_ValidIds_ShouldSubmitAsyncDelete() {
//        List<Long> ids = Collections.singletonList(TEST_SECRET_KEY_ID);
//        SecretKey secretKey = new SecretKey();
//        secretKey.setFileName(TEST_FILE_NAME);
//        when(secretServiceMapper.selectBatchIds(ids)).thenReturn(Collections.singletonList(secretKey));
//
//        secretKeyService.deleteList(ids);
//
//        // 验证异步任务提交
//        verify(secretKeyService.FILE_DELETE_EXECUTOR, times(1)).execute(any(Runnable.class));
//        verify(secretServiceMapper, times(1)).deleteBatchIds(ids);
//    }
//
//    // ======================= 异常处理测试 =======================
//    @Test
//    void testDeleteSecretKey_FileDeleteFailed_ShouldThrowException() {
//        aliOssUtil.deleteFile(TEST_FILE_NAME);
//        SecretKey secretKey = new SecretKey();
//        secretKey.setId(TEST_SECRET_KEY_ID);
//        secretKey.setFileName(TEST_FILE_NAME);
//        when(secretServiceMapper.selectById(TEST_SECRET_KEY_ID)).thenReturn(secretKey);
//
//        assertThrows(ServiceException.class, () -> {
//            secretKeyService.deleteSecretKey(TEST_SECRET_KEY_ID);
//        }, "删除文件失败");
//    }
//}