package cd.casic.module.process.process.setting;


import cd.casic.ci.commons.bean.process.setting.AuthThird;
import cd.casic.ci.commons.bean.process.setting.AuthThirdQuery;
import cd.casic.framework.commons.pojo.PageResult;

import java.util.List;

/**
 * 流水线第三方认证服务接口
 */
public interface AuthThirdService {

    /**
     * 创建流水线第三方授权
     *
     * @param authThird 流水线第三方授权
     * @return 流水线第三方授权id
     */
    int createAuthServer(AuthThird authThird);

    /**
     * 删除流水线第三方授权
     * @param authServerId 流水线第三方授权id
     */
    void deleteAuthServer(String authServerId);

    /**
     * 更新第三方授权信息
     * @param authThird 信息
     */
    void updateAuthServer(AuthThird authThird);

    /**
     * 查询第三方授权信息
     * @param authServerId id
     * @return 信息
     */
    AuthThird findOneAuthServer(String authServerId);

    /**
     * 查询流水线第三方授权
     * @param thirdQuery 查询条件
     * @return 流水线第三方授权列表
     */
    List<AuthThird> findAuthServerList(AuthThirdQuery thirdQuery);

    /**
     * 分页查询流水线第三方授权
     * @param thirdQuery 查询条件
     * @return 分页流水线第三方授权
     */
    PageResult<AuthThird> findAuthServerPage(AuthThirdQuery thirdQuery);

    /**
     * 查询所有流水线第三方授权
     * @return 流水线第三方授权列表
     */
    List<AuthThird> findAllAuthServer();


    /**
     * 根据ID列表批量查询流水线第三方授权
     * @param idList 流水线第三方授权ID列表
     * @return 流水线第三方授权列表
     */
    List<AuthThird> findAllAuthServerList(List<String> idList);

    /**
     * 查询流水线第三方授权数量
     * @return 流水线第三方授权数量
     */
    Integer findAuthServerNumber();
    
    
}
