package cd.casic.module.process.process.setting.impl;

import cd.casic.ci.commons.bean.commons.PipelineConstants;
import cd.casic.ci.commons.bean.process.setting.AuthHostGroup;
import cd.casic.ci.commons.bean.process.setting.AuthHostGroupDetails;
import cd.casic.ci.commons.bean.process.setting.AuthHostGroupQuery;
import cd.casic.ci.commons.bean.process.setting.HostGroup;
import cd.casic.ci.commons.bean.utils.PipelineUtil;
import cd.casic.module.process.convert.setting.AuthHostGroupConvert;
import cd.casic.module.process.process.setting.AuthHostGroupDetailsService;
import cd.casic.module.process.process.setting.AuthHostGroupService;
import cd.casic.module.process.process.setting.dao.AuthHostGroupDao;
import cd.casic.module.process.process.setting.dataobject.AuthHostGroupDo;
import cn.hutool.core.util.IdUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Service
public class AuthHostGroupServiceImpl implements AuthHostGroupService {


    @Autowired
    AuthHostGroupDao authHostGroupDao;

    @Autowired
    AuthHostGroupDetailsService groupDetailsService;

    @Override
    public String creatHostGroup(AuthHostGroup hostGroup) {
        String date = PipelineUtil.date(1);
        hostGroup.setCreateTime(date);
        authHostGroupDao.insert(AuthHostGroupConvert.INSTANCE.convertDo(hostGroup));

        List<AuthHostGroupDetails> detailsList = hostGroup.getDetailsList();
        if (Objects.isNull(detailsList) || detailsList.isEmpty()) {
            return hostGroup.getGroupId();
        }

        // 创建关联关系
        for (AuthHostGroupDetails hostGroupDetails : detailsList) {
            hostGroupDetails.setGroupId(hostGroup.getGroupId());
            groupDetailsService.creatHostGroupDetails(hostGroupDetails);
        }

        return hostGroup.getGroupId();
    }

    @Override
    public void updateHostGroup(AuthHostGroup hostGroup) {
        String groupId = hostGroup.getGroupId();
        // 删除旧的关联关系
        List<AuthHostGroupDetails> hostGroupDetailsList = groupDetailsService.findHostGroupDetailsList(groupId);
        for (AuthHostGroupDetails authHostGroupDetails : hostGroupDetailsList) {
            String id = authHostGroupDetails.getId();
            groupDetailsService.deleteHostGroupDetails(id);
        }

        // 创建新的关联关系
        List<AuthHostGroupDetails> detailsList = hostGroup.getDetailsList();
        for (AuthHostGroupDetails hostGroupDetails : detailsList) {
            hostGroupDetails.setGroupId(groupId);
            groupDetailsService.creatHostGroupDetails(hostGroupDetails);
        }
        authHostGroupDao.updateById(AuthHostGroupConvert.INSTANCE.convertDo(hostGroup));
    }

    @Override
    public void deleteHostGroup(String groupId) {
        List<AuthHostGroupDetails> hostGroupDetailsList = groupDetailsService.findHostGroupDetailsList(groupId);
        for (AuthHostGroupDetails authHostGroupDetails : hostGroupDetailsList) {
            String id = authHostGroupDetails.getId();
            groupDetailsService.deleteHostGroupDetails(id);
        }
        authHostGroupDao.deleteById(groupId);
    }

    @Override
    public AuthHostGroup findOneHostGroup(String groupId) {
        AuthHostGroup oneHostGroup = AuthHostGroupConvert.INSTANCE.convert(authHostGroupDao.selectById(groupId));
        if (Objects.isNull(oneHostGroup)) {
            return null;
        }
        List<AuthHostGroupDetails> hostGroupDetailsList = groupDetailsService.findHostGroupDetailsList(groupId);
        oneHostGroup.setDetailsList(hostGroupDetailsList);
        return oneHostGroup;
    }

    @Override
    public List<HostGroup> findOneHostGroupByGroup(String groupId, Integer strategyNumber, String strategyType) {
        List<AuthHostGroupDetails> detailsList = groupDetailsService.findHostGroupDetailsList(groupId);

        if (strategyType.equals(PipelineConstants.DEFAULT)) {
            return null;
        }

        int size = detailsList.size();
        if (size <= strategyNumber) {
            return null;
        }

        List<HostGroup> resultList = new ArrayList<>();

        for (int i = 0; i < detailsList.size(); i += strategyNumber) {
            List<AuthHostGroupDetails> sublist = detailsList.subList(i, Math.min(i + strategyNumber, detailsList.size()));
            String uuid = IdUtil.fastUUID();
            resultList.add(new HostGroup(uuid, sublist));
        }
        return resultList;
    }


    @Override
    public List<AuthHostGroup> findAllHostGroup() {
        List<AuthHostGroupDo> authHostGroupDos = authHostGroupDao.selectList();
        return AuthHostGroupConvert.INSTANCE.convertList(authHostGroupDos);

    }

    @Override
    public List<AuthHostGroup> findHostGroupList(AuthHostGroupQuery groupQuery) {
        List<AuthHostGroupDo> authHostGroupDos = authHostGroupDao.selectList(AuthHostGroupDo::getCreator, groupQuery.getUserId());
        List<AuthHostGroup> authHostGroups = AuthHostGroupConvert.INSTANCE.convertList(authHostGroupDos);
        for (AuthHostGroup authHostGroup : authHostGroups) {
            String groupId = authHostGroup.getGroupId();
            List<AuthHostGroupDetails> hostGroupDetailsList = groupDetailsService.findHostGroupDetailsList(groupId);
            authHostGroup.setDetailsList(hostGroupDetailsList);
        }
        return authHostGroups;
    }

    @Override
    public Long findHostGroupNumber() {
        return authHostGroupDao.selectCount();
    }

}
