package cd.casic.module.process.task.code.service.impl;


/**
 * @ClassName TaskArtifactXpackServiceImpl
 * @Author hopeli
 * @Date 2025/5/2 20:14
 * @Version 1.0
 */


import cd.casic.ci.commons.bean.process.setting.AuthThird;
import cd.casic.ci.commons.annotation.rpc.Exporter;
import cd.casic.module.process.process.setting.AuthService;
import cd.casic.module.process.process.setting.AuthThirdService;
import cd.casic.module.process.task.code.dao.TaskCodeDao;
import cd.casic.module.process.task.code.dataobject.TaskCodeDo;
import cd.casic.module.process.task.code.model.TaskCode;
import cd.casic.module.process.task.code.model.ThirdHouse;
import cd.casic.module.process.task.code.model.ThirdQuery;
import cd.casic.module.process.task.code.service.TaskCodeGitHubService;
import cd.casic.module.process.task.code.service.TaskCodeGitLabService;
import cd.casic.module.process.task.code.service.TaskCodeGiteeService;
import cd.casic.module.process.task.code.service.TaskCodeGittokService;
import cd.casic.module.process.task.code.service.TaskCodePriGitLabService;
import cd.casic.module.process.task.code.service.TaskCodeService;
import cd.casic.module.process.toolkit.beans.BeanMapper;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

import static cd.casic.ci.commons.bean.commons.PipelineConstants.AUTH_NONE;
import static cd.casic.ci.commons.bean.commons.PipelineConstants.AUTH_USER_PASS;
import static cd.casic.ci.commons.bean.commons.PipelineConstants.TASK_CODE_PRI_GITLAB;
import static cd.casic.module.process.support.util.util.PipelineFinal.TASK_CODE_GITEE;
import static cd.casic.module.process.support.util.util.PipelineFinal.TASK_CODE_GITHUB;
import static cd.casic.module.process.support.util.util.PipelineFinal.TASK_CODE_GITLAB;
import static cd.casic.module.process.support.util.util.PipelineFinal.TASK_CODE_SVN;
import static cd.casic.module.process.support.util.util.PipelineFinal.TASK_CODE_XCODE;


@Service
@Exporter
public class TaskCodeServiceImpl implements TaskCodeService {

    @Resource
    TaskCodeDao taskCodeDao;

    @Resource
    AuthService authServer;

    @Resource
    AuthThirdService authServerServer;

    @Resource
    TaskCodeGiteeService giteeService;

    @Resource
    TaskCodeGitHubService gitHubService;

    @Resource
    TaskCodeGitLabService gitLabService;

    @Resource
    TaskCodePriGitLabService priGitLabService;

    @Resource
    TaskCodeGittokService xcodeService;

    @Override
    public String createCode(TaskCode taskCode) {
        return taskCodeDao.createCode(BeanMapper.map(taskCode, TaskCodeDo.class));
    }

    @Override
    public TaskCode findCodeByAuth(String taskId){
        TaskCodeDo oneCodeEntity = taskCodeDao.findOneCode(taskId);
        TaskCode taskCode = BeanMapper.map(oneCodeEntity, TaskCode.class);
        String authId = taskCode.getAuthId();

        if (Objects.isNull(authId)){
            return taskCode;
        }
        taskCode.setAuth(findAuth(taskCode.getAuthId()));
        return taskCode;
    }

    @Override
    public Boolean codeValid(String taskType,TaskCode code){
        switch (taskType) {
            case TASK_CODE_GITEE, TASK_CODE_GITLAB, TASK_CODE_GITHUB,TASK_CODE_PRI_GITLAB -> {
                if (StringUtils.isEmpty(code.getHouseId())) {
                    return false;
                }
                return !StringUtils.isEmpty(code.getAuthId());
            }
            case TASK_CODE_XCODE -> {
                if (StringUtils.isEmpty(code.getHouseId())) {
                    return false;
                }
                return !StringUtils.isEmpty(code.getAuthId());
            }
            case TASK_CODE_SVN -> {
                // String svnFile = code.getSvnFile();
                // return !StringUtils.isEmpty(svnFile);
                return true;
            }
            default -> {
                String codeAddress = code.getCodeAddress();
                return !StringUtils.isEmpty(codeAddress);
            }
        }
    }

    @Override
    public void deleteCode(String codeId) {
        taskCodeDao.deleteCode(codeId);
    }

    @Override
    public void updateCode(TaskCode taskCode) {
        TaskCodeDo oneCodeEntity = taskCodeDao.findOneCode(taskCode.getTaskId());
        TaskCode oneCode = BeanMapper.map(oneCodeEntity, TaskCode.class);
        String authId = oneCode.getAuthId();
        String houseId = taskCode.getHouseId();

        if (!StringUtils.isEmpty(houseId)){
            ThirdHouse storeHouse = null;
            ThirdQuery thirdQuery = new ThirdQuery();
            thirdQuery.setHouseId(houseId);
            thirdQuery.setAuthId(authId);
            switch (taskCode.getType()) {
                case TASK_CODE_GITEE  -> {
                    storeHouse = giteeService.findStoreHouse(thirdQuery);
                }
                case TASK_CODE_GITHUB -> {
                    storeHouse = gitHubService.findStoreHouse(thirdQuery);
                }
                case TASK_CODE_GITLAB -> {
                    storeHouse = gitLabService.findStoreHouse(thirdQuery);
                }
                case TASK_CODE_PRI_GITLAB -> {
                    storeHouse = priGitLabService.findStoreHouse(thirdQuery);
                }
                case TASK_CODE_XCODE -> {
                    storeHouse = xcodeService.findStoreHouse(thirdQuery);
                }
                default -> {
                }
            }
            if (!Objects.isNull(storeHouse)){
                taskCode.setCodeAddress(storeHouse.getHouseWebUrl());
                taskCode.setCodeName(storeHouse.getNameWithSpace());
                taskCode.setCodeBranch(storeHouse.getDefaultBranch());
            }
        }else {
            taskCode.setCodeAddress(taskCode.getCodeName());
        }

        String authType = taskCode.getAuthType();
        if (!StringUtils.isEmpty(authType)){
            if (authType.equals(AUTH_NONE)){
                taskCode.setUsername("");
                taskCode.setPassword("");
                taskCode.setPriKey("");
            }else if (authType.equals(AUTH_USER_PASS)){
                taskCode.setPriKey("");
            }else {
                taskCode.setUsername("");
                taskCode.setPassword("");
            }
        }
        TaskCodeDo taskCodeDo = BeanMapper.map(taskCode, TaskCodeDo.class);
        taskCodeDao.updateCode(taskCodeDo);
    }

    @Override
    public void updateOneCode(TaskCode taskCode){
        taskCodeDao.updateCode(BeanMapper.map(taskCode, TaskCodeDo.class));
    }

    @Override
    public TaskCode findOneCode(String codeId) {
        TaskCodeDo oneCodeEntity = taskCodeDao.findOneCode(codeId);
        return BeanMapper.map(oneCodeEntity, TaskCode.class);
    }

    @Override
    public List<TaskCode> findAllCode() {
        List<TaskCodeDo> allCode = taskCodeDao.findAllCode();
        List<TaskCode> taskCodes = BeanMapper.mapList(allCode, TaskCode.class);
        if (taskCodes == null){
            return null;
        }
        for (TaskCode taskCode : taskCodes) {
            taskCode.setAuth(findAuth(taskCode.getAuthId()));
        }
        return taskCodes;
    }

    @Override
    public List<TaskCode> findAllCodeList(List<String> idList) {
        return BeanMapper.mapList(taskCodeDao.findAllCodeList(idList), TaskCode.class);
    }

    //获认证信息
    private Object findAuth(String id){
        AuthThird oneAuthServer = authServerServer.findOneAuthServer(id);
        if (oneAuthServer != null){
           return oneAuthServer;
        }
        return authServer.findOneAuth(id);
    }

}


