package cd.casic.module.process.task.codescan.controller;


/**
 * @ClassName TaskArtifactXpackServiceImpl
 * @Author hopeli
 * @Date 2025/5/3 20:15
 * @Version 1.0
 */

import cd.casic.ci.commons.bean.commons.Pagination;
import cd.casic.ci.commons.bean.utils.Result;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.module.process.task.code.service.SpotbugsScanService;
import cd.casic.module.process.task.codescan.model.SpotbugsBugInstance;
import cd.casic.module.process.task.codescan.model.SpotbugsBugPackageStats;
import cd.casic.module.process.task.codescan.model.SpotbugsBugQuery;
import cd.casic.module.process.task.codescan.model.SpotbugsBugSummary;
import cd.casic.module.process.task.codescan.service.impl.SpotbugsXmlConfig;
import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @pi.protocol: http
 * @pi.groupName: 流水线代码扫描Spotbugs控制器
 */
@RestController
@RequestMapping("/spotbugsScan")
public class SpotbugsScanController {

    @Resource
    SpotbugsScanService spotbugsScanService;

    /**
     * @pi.name:条件查询代码扫描结果
     * @pi.url:/spotbugsScan/findScanBugsList
     * @pi.methodType:post
     * @pi.request-type:json
     * @pi.param: model=spotbugsBugQuery
     */
    @RequestMapping(path = "/findScanBugsList", method = RequestMethod.POST)
    public Result<List<SpotbugsBugSummary>> findSpotbugsList(@RequestBody @Valid @NotNull SpotbugsBugQuery spotbugsBugQuery) {
        List<SpotbugsBugSummary> spotbugsList = spotbugsScanService.findSpotbugsList(spotbugsBugQuery);

        return Result.ok(spotbugsList);
    }

    /**
     * @pi.name:分页查询代码扫描结果
     * @pi.url:/spotbugsScan/findSpotbugsPage
     * @pi.methodType:post
     * @pi.request-type:json
     * @pi.param: model=spotbugsBugQuery
     */
    @RequestMapping(path = "/findSpotbugsPage", method = RequestMethod.POST)
    public Result<Pagination<SpotbugsBugSummary>> findSpotbugsPage(@RequestBody @Valid @NotNull SpotbugsBugQuery spotbugsBugQuery) {

        PageResult<SpotbugsBugSummary> spotbugsList = spotbugsScanService.findSpotbugsPage(spotbugsBugQuery);

        return Result.ok(spotbugsList);
    }

    /**
     * @pi.name:查询代码扫描详情
     * @pi.url:/spotbugsScan/findBugs
     * @pi.methodType:post
     * @pi.request-type: formdata
     * @pi.param: name=xmlPath;dataType=string;value=xmlPath;
     */
    @RequestMapping(path = "/findBugs", method = RequestMethod.POST)
    public Result<List<SpotbugsBugInstance>> findAllRepository(@NotNull String xmlPath) {
        List<SpotbugsBugInstance> scanBugs = new SpotbugsXmlConfig().findScanBugs(xmlPath);

        return Result.ok(scanBugs);
    }


    /**
     * @pi.name:删除代码扫描结果
     * @pi.url:/spotbugsScan/deleteSpotbugs
     * @pi.methodType:post
     * @pi.request-type: formdata
     * @pi.param: name=bugId;dataType=string;value=bugId;
     */
    @RequestMapping(path = "/deleteSpotbugs", method = RequestMethod.POST)
    public Result<List<SpotbugsBugPackageStats>> deleteSpotbugs(@NotNull String bugId) {

        spotbugsScanService.deleteSpotbugs(bugId);
        return Result.ok();
    }












}
