package cd.casic.module.process.toolkit.beans;

import cd.casic.ci.commons.bean.exception.SystemException;
import cd.casic.module.process.core.util.BeanUtils;
import cd.casic.module.process.toolkit.beans.metadata.BeanMapperRegister;
import cd.casic.module.process.toolkit.beans.model.BeanMapping;
import cd.casic.module.process.toolkit.beans.model.FieldMapping;
import ognl.Ognl;
import ognl.OgnlException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class BeanMapper {
    private static Logger logger = LoggerFactory.getLogger(BeanMapper.class);

    public BeanMapper() {
    }

    public static <T> T map(Object sourceObject, Class<T> targetClz) {
        if (sourceObject == null) {
            return null;
        } else if (targetClz == null) {
            throw new SystemException("targetClass must not be null.");
        } else {
            try {
                Class sourceClz = sourceObject.getClass();
                BeanMapping beanMapping = BeanMapperRegister.getBeanMapping(sourceClz, targetClz);
                if (beanMapping == null) {
                    throw new SystemException(String.format("bean mapper not found,source:%s,target:%s.", sourceClz.getName(), targetClz.getName()));
                } else {
                    T targetObject = targetClz.newInstance();
                    List<FieldMapping> propertyMappingList = beanMapping.getPropertyMappingList();
                    Iterator var6 = propertyMappingList.iterator();

                    while(var6.hasNext()) {
                        FieldMapping propertyMapping = (FieldMapping)var6.next();
                        String source = propertyMapping.getSource();
                        String target = propertyMapping.getTarget();

                        try {
                            Object fieldValue;
                            if (isComplexField(source)) {
                                fieldValue = getParentFiled(sourceObject, source);
                                if (fieldValue == null) {
                                    continue;
                                }
                            }

                            fieldValue = Ognl.getValue(source, sourceObject);
                            if (fieldValue != null) {
                                if (isComplexField(target)) {
                                    initParentFieldIfNull(targetObject, target);
                                }

                                Ognl.setValue(target, targetObject, fieldValue);
                            }
                        } catch (OgnlException var11) {
                            throw new SystemException("表达式解析失败");
                        }
                    }

                    return targetObject;
                }
            } catch (InstantiationException var12) {
//                throw new SystemException(var12);
                throw new SystemException("系统错误");
            } catch (IllegalAccessException var13) {
//                throw new SystemException(var13);
                throw new SystemException("系统错误");
            }
        }
    }

    static boolean isComplexField(String exp) {
        return exp.contains(".");
    }

    static Object getParentFiled(Object sourceObject, String exp) {
        String[] arr = exp.split("\\.");
        String fieldName = arr[0];
        PropertyDescriptor fieldPd = BeanUtils.getPropertyDescriptor(sourceObject.getClass(), fieldName);
        Method fieldReadMethod = fieldPd.getReadMethod();
        if (!Modifier.isPublic(fieldReadMethod.getDeclaringClass().getModifiers())) {
            fieldReadMethod.setAccessible(true);
        }

        try {
            Object fieldValue = fieldReadMethod.invoke(sourceObject);
            return fieldValue;
        } catch (IllegalAccessException var7) {
//            throw new SystemException(var7);
            throw new SystemException("系统错误");
        } catch (InvocationTargetException var8) {
//            throw new SystemException(var8);
            throw new SystemException("系统错误");
        }
    }

    static void initParentFieldIfNull(Object targetObject, String exp) {
        if (exp.contains(".")) {
            String[] arr = exp.split("\\.");
            String fieldName = arr[0];

            try {
                PropertyDescriptor fieldPd = BeanUtils.getPropertyDescriptor(targetObject.getClass(), fieldName);
                Method fieldReadMethod = fieldPd.getReadMethod();
                if (!Modifier.isPublic(fieldReadMethod.getDeclaringClass().getModifiers())) {
                    fieldReadMethod.setAccessible(true);
                }

                Object fieldValue = fieldReadMethod.invoke(targetObject);
                if (fieldValue == null) {
                    Class fieldType = fieldPd.getPropertyType();
                    fieldValue = fieldType.newInstance();
                    if (fieldValue == null) {
                        throw new SystemException(String.format("init field value error.", fieldName));
                    } else {
                        Method writeMethod = fieldPd.getWriteMethod();
                        if (!Modifier.isPublic(writeMethod.getDeclaringClass().getModifiers())) {
                            writeMethod.setAccessible(true);
                        }

                        writeMethod.invoke(targetObject, fieldValue);
                    }
                }
            } catch (IllegalAccessException var9) {
//                throw new SystemException(var9);
                throw new SystemException("系统错误");
            } catch (InvocationTargetException var10) {
//                throw new SystemException(var10);
                throw new SystemException("系统错误");
            } catch (InstantiationException var11) {
//                throw new SystemException(var11);
                throw new SystemException("系统错误");
            }
        }
    }

    public static <T> List<T> mapList(List<?> sourceObjectList, Class<T> targetClz) {
        List<T> targetList = new ArrayList();
        if (sourceObjectList != null && sourceObjectList.size() != 0) {
            try {
                Iterator var3 = sourceObjectList.iterator();

                while(var3.hasNext()) {
                    Object sourceObject = var3.next();
                    T targetObject = map(sourceObject, targetClz);
                    targetList.add(targetObject);
                }

                return targetList;
            } catch (Exception var6) {
//                throw new SystemException(var6);
                throw new SystemException("系统错误");
            }
        } else {
            return targetList;
        }
    }
}
